@@ -95,7 +95,7 @@ try:
 except:
 	has_numpy = False
 
-__version__ = '1.0.2'
+__version__ = '1.0.3'
 
 def NoneStr(obj): return 'NULL'
 
@@ -206,7 +206,7 @@ class R: # (object):
 	'''
 	A Python class to enclose an R process.
 	'''
-	__Rfun = '''.getRvalue4Python__ <- function(x, use_dict=FALSE) {
+	__Rfun = r'''.getRvalue4Python__ <- function(x, use_dict=NULL) {
 	has_numpy <- %s
 	if (has_numpy) {
 		headstr <- 'numpy.array('
@@ -278,24 +278,32 @@ class R: # (object):
 		if (!is.null(nms) &&  length(nms)>0) {
 			nms <- paste('"', nms, '"', sep='')
 			x <- sapply(seq(nms), function(i) paste('(', nms[i], ',', x[i], ')') ) 
-			if (use_dict) x <- paste('dict([', paste(x, collapse=', '), '])', sep='')
-			else x <- paste('[', paste(x, collapse=', '), ']', sep='')
+			if (identical(use_dict, TRUE)) x <- paste('dict([', paste(x, collapse=', '), '])', sep='')
+			else if (identical(use_dict, FALSE))  x <- paste('[', paste(x, collapse=', '), ']', sep='')
+			else { # should be NULL or something else
+				if (length(nms) != length(unique(nms))) x <- paste('[', paste(x, collapse=', '), ']', sep='') 
+				else x <- paste('dict([', paste(x, collapse=', '), '])', sep='')
+				}
 			}
 		else
 			x <- paste('[', paste(x, collapse=', '), ']', sep='')
 		return(x) }
-	Str4Py <- function(x) {
+	Str4Py <- function(x, outmost=FALSE) {
 		# no considering on NA, Inf, ...
 		# use is.XXX, typeof, class, mode, storage.mode, sprintf
 		if (is.factor(x)) x <- as.vector(x)
-		if (is.null(x)) return(NullStr(x))
-		if (is.vector(x) && !is.list(x)) return(VectorStr(x))
-		if (is.matrix(x) || is.array(x)) return(ArrayStr(x))
-		if (is.data.frame(x)) return(DataFrameStr(x))
-		if (is.list(x)) return(ListStr(x))
-		return(Str4Py(as.character(x))) # other objects will be convert to character, or use "gettext"
-		return(NullStr(x)) }
-	Str4Py(x)
+		rlt <- {
+			if (is.null(x)) NullStr(x)
+			else if (is.vector(x) && !is.list(x)) VectorStr(x)
+			else if (is.matrix(x) || is.array(x)) ArrayStr(x)
+			else if (is.data.frame(x)) DataFrameStr(x)
+			else if (is.list(x)) ListStr(x)
+			else Str4Py(as.character(x)) # other objects will be convert to character (instead of NullStr), or use "gettext"
+			}
+		if (outmost) rlt <- gsub('\\\\', '\\\\\\\\', rlt) 
+		return(rlt)
+		}
+	Str4Py(x, outmost=TRUE)
 	}
 	# initalize library path for TCL/TK based environment on Windows, e.g. Python IDLE
 	.addLibs <- function() {
@@ -304,8 +312,13 @@ class R: # (object):
 		libpaths <- .libPaths()
 		for (apath in userpath) {
 			if (length(grep(apath, libpaths)) > 0) next
-			apath <- paste(ruser, '/Documents', substr(apath, nchar(ruser)+1, nchar(apath)), sep='')
-			.libPaths(apath)
+			if (file.exists(apath)) .libPaths(apath)
+			else {
+				d <- '/Documents'
+				if (substr(ruser, nchar(ruser)-nchar(d)+1, nchar(ruser)) != d) {
+					apath <- paste(ruser,d, substr(apath, nchar(ruser)+1, nchar(apath)), sep='')
+					if (file.exists(apath)) .libPaths(apath)}
+				}
 			}
 		}
 	if(identical(.Platform$OS.type, 'windows')) .addLibs()
@@ -313,7 +326,7 @@ class R: # (object):
 	''' 
 	_DEBUG_MODE = True
 
-	def __init__(self, RCMD='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, use_dict=False, host='localhost', user=None, ssh='ssh', return_err=True):
+	def __init__(self, RCMD='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, use_dict=None, host='localhost', user=None, ssh='ssh', return_err=True):
 		'''
 		RCMD: The name of a R interpreter, path information should be included
 			if it is not in the system search path.
@@ -325,8 +338,11 @@ class R: # (object):
 			longer than this value.
 		use_numpy: Used as a boolean value. A False value will disable numpy
 			even if it has been imported.
-		use_dict: named list will be returned a dict if use_dict is True,
-			otherwise it will be a list of tuples (name, value)
+		use_dict: A R named list will be returned as a Python dictionary if
+			"use_dict" is True, or a list of tuples (name, value) if "use_dict"
+			is False. If "use_dict" is None, the return value will be a
+			dictionary if there is no replicated names, or a list if replicated
+			names found.
 		host: The computer name (or IP) on which the R interpreter is
 			installed. The value "localhost" means that R locates on the the
 			localhost computer. On POSIX systems (including Cygwin environment
@@ -405,7 +421,11 @@ class R: # (object):
 				rltonce = recv_some(self.prog, e=1, t=self.wait)
 				if rltonce: rlt = rlt + rltonce
 			except: break
-		else: rlt = rlt[:-len(rlt_tail)]
+		else: 
+			if rlt.endswith(rlt_tail):
+				rlt = rlt[:-len(rlt_tail)]
+			elif rlt.endswith(rlt_tail1):
+				rlt = rlt[:-len(rlt_tail1)]
 		if fn is not None:
 			os.unlink(fn)
 		return rlt
@@ -440,12 +460,16 @@ class R: # (object):
 			raise RError('Leading underscore ("_") is not permitted in R variable names!')
 		use_try = use_try or self._DEBUG_MODE
 		if use_dict is None: use_dict = self.use_dict
-		cmd = '.getRvalue4Python__(%s, use_dict=%s)' % (obj, use_dict and 'TRUE' or 'FALSE')
+		cmd = '.getRvalue4Python__(%s, use_dict=%s)' % (obj, use_dict is None and 'NULL' or use_dict and 'TRUE' or 'FALSE')
 		rlt = self.__call__(cmd, use_try=use_try)
-		head = len((use_try and 'try({%s})%s[1] ' or '%s%s[1] ') % (cmd, self.tail)) 
-		tail = len(rlt) - len(self.tail) # - len('"')
+		head = (use_try and 'try({%s})%s[1] ' or '%s%s[1] ') % (cmd, self.tail) 
+		# sometimes (e.g. after "library(fastICA)") the R on Windows uses '\n' instead of '\r\n'
+		head = rlt.startswith(head) and len(head) or len(head) - 1 
+		tail = rlt.endswith(self.tail) and len(rlt) - len(self.tail) or len(rlt) - len(self.tail) + 1 # - len('"')
 		try:
-			rlt = eval(eval(rlt[head:tail]))
+			rlt = eval(eval(rlt[head:tail])) # The inner eval remove quotes and recover escaped characters.
+			#head, tail = head + 1, tail - 1 # to remove quote
+			#rlt = eval(rlt[head:tail])
 		except:
 			raise RError(rlt)
 		return rlt
@@ -520,7 +544,7 @@ class R: # (object):
 
 
 # for a single-round duty:
-def runR(CMDS, Robj='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, use_dict=False, host='localhost', user=None, ssh='ssh'):
+def runR(CMDS, Robj='R', wait0=2, wait=0.01, max_len=1000, use_numpy=True, use_dict=None, host='localhost', user=None, ssh='ssh'):
 	'''
 	Run a (list of) R command(s), and return the output from the STDOUT.
 
